/******************************************************************************
 *{@C
 *      Copyright:      2006-2018 Paul Obermeier (obermeier@tcl3d.org)
 *
 *                      See the file "Tcl3D_License.txt" for information on
 *                      usage and redistribution of this file, and for a
 *                      DISCLAIMER OF ALL WARRANTIES.
 *
 *      Module:         Tcl3D -> tcl3dFTGL
 *      Filename:       tcl3dFTGL.i
 *
 *      Author:         Paul Obermeier
 *
 *      Description:    Main SWIG file for wrapping the Freetype Fonts for 
 *                      OpenGL Library (FTGL).
 *                      This file is responsible for the Tcl package tcl3dFTGL.
 *
 *                      The macro USE_FTGL_IF must be defined at compilation
 *                      time to include the functionality of the FTGL library
 *                      into the Tcl3D package.
 *
 *****************************************************************************/

// The name of our generated Tcl package.

%module tcl3dFTGL


// Code to initialize the Tk stubs library.

%{
    #include <tk.h>
%}

%init %{
    #ifndef USE_FTGL_IF
        Tcl_SetResult(interp, (char *) "FTGL library not wrapped", TCL_STATIC);
        return TCL_ERROR;
    #endif

    #ifdef USE_TK_STUBS
        if (Tk_InitStubs(interp, (char*)"8.5", 0) == NULL) {
            return TCL_ERROR;
        }
    #endif
%}

#ifdef USE_FTGL_IF

// Standard modules from the SWIG library

%include typemaps.i


// Common Tcl3D wrapper modules.

%include tcl3dVectors.i
%include tcl3dConstHash.i
%include tcl3dPointer.i
%include tcl3dArgv.i
%include tcl3dTypemaps.i


// Wrap the FTGL library.

%include ftgl.i

#endif // USE_FTGL_IF
